/*
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *  
 */

/*
 * Created on Feb 14, 2005
 *
 */
package com.sunlabs.archsim.tests.actionarbiter;

import com.sunlabs.archsim.gasp.State;;

/**
 * @author ib27688
 *
 */
public class AATFactory {
	
	private AATFactory() {}
	
	public static ActionArbiterTest please(String name) {
		
		ActionArbiterTest test= new ActionArbiterTest(name);	
		
		// make the source A
		AATSource srcA= new AATSource(
				AATGlobal.SOURCE_ACTION_A_NAME, test, 0);
		
		// make the terminals that attach to the source
		AATOutput dataOutA= new AATOutput(
				AATGlobal.DATA_OUT_TERMINAL_NAME, test);
		
		AATInput ackDataOutA= new AATInput(
				AATGlobal.ACK_DATA_OUT_TERMINAL_NAME, test);
		
		// attache the terminals to the source action
		srcA.attach(dataOutA);
		srcA.attach(ackDataOutA);
		
		// now make the states between the source A and
		// the action AC
		State dataA= new State(AATGlobal.DATA_IN_A_STATE_NAME, test);
		State ackDataA= new State(AATGlobal.ACK_DATA_IN_A_STATE_NAME, test);
		
		// attach terminals to these states
		dataA.attach(dataOutA);
		ackDataA.attach(ackDataOutA);		
		
		//-----------------
		// make action from source A to sink (C)...
		AATAction actionAC= new AATAction(
				AATGlobal.ACTION__AC_NAME, test);

		
		// make terminals on the data source side of action AC
		AATInput dataInAC= new AATInput(
				AATGlobal.DATA_IN_TERMINAL_NAME, test);
		// AATInput dataInAC= new AATInput(
		//				AATGlobal.DATA_IN_TERMINAL_NAME, test);
		
		AATOutput ackDataInAC= new AATOutput(
				AATGlobal.ACK_DATA_IN_TERMINAL_NAME, test);
		
		// attach the terminals to the action
		actionAC.attach(dataInAC);
		actionAC.attach(ackDataInAC);
		
		
		// attach the terminals to the states
		dataA.attach(dataInAC);
		ackDataA.attach(ackDataInAC);

	
		//------------------
		// make the source B
		AATSource srcB= new AATSource(
				AATGlobal.SOURCE_ACTION_B_NAME, test, 1000);
		
		// make the terminals that attach to the source
		AATOutput dataOutB= new AATOutput(
				AATGlobal.DATA_OUT_TERMINAL_NAME, test);
		
		AATInput ackDataOutB= new AATInput(
				AATGlobal.ACK_DATA_OUT_TERMINAL_NAME, test);
		
		// attache the terminals to the source action
		srcB.attach(dataOutB);
		srcB.attach(ackDataOutB);
		
		// now make the states between the source B and
		// the action BC
		State dataB= new State(AATGlobal.DATA_IN_B_STATE_NAME, test);
		
		State ackDataB= new State(
				AATGlobal.ACK_DATA_IN_B_STATE_NAME, test);
		
		// attach terminals to these stated
		dataB.attach(dataOutB);
		ackDataB.attach(ackDataOutB);		

		
		//-------------------
		// make action from source B to sink (C)...
		AATAction actionBC= new AATAction(
				AATGlobal.ACTION_BC_NAME, test);
		
		// make terminals on the data source side of action BC
		AATInput dataInBC= new AATInput(
				AATGlobal.DATA_IN_TERMINAL_NAME, test);
		
		AATOutput ackDataInBC= new AATOutput(
				AATGlobal.ACK_DATA_IN_TERMINAL_NAME, test);
		
		// attach the terminals to the action
		actionBC.attach(dataInBC);
		actionBC.attach(ackDataInBC);
		
		// attach the terminals to the states
		dataB.attach(dataInBC);
		ackDataB.attach(ackDataInBC);
		
		// connect the two arbitration actions. that is, make them
		// peers so that only one at a time can fire
		actionAC.attachPeer(actionBC);
		
		// now make the terminals on the sink end of action AC
		dataOutA= new AATOutput(AATGlobal.DATA_OUT_TERMINAL_NAME, test);
		
		ackDataOutA= new AATInput(AATGlobal.ACK_DATA_OUT_TERMINAL_NAME, test);
		
		// attach the terminals to the action
		actionAC.attach(dataOutA);
		actionAC.attach(ackDataOutA);
		
		// set the "done input" = the arbiter "done" signal
		actionAC.setDoneInput(ackDataOutA);
		
		// make the states that attach to these terminals
		State dataC= new State(AATGlobal.DATA_OUT_C_STATE_NAME, test);
		
		State ackDataC= new State(AATGlobal.ACK_DATA_OUT_C_STATE_NAME, test);
		
		// attach terminals to the states
		dataC.attach(dataOutA);
		ackDataC.attach(ackDataOutA);
		
		// now make the terminals on the sink end of action BC
		dataOutB= new AATOutput(AATGlobal.DATA_OUT_TERMINAL_NAME, test);
		
		ackDataOutB= new AATInput(
				AATGlobal.ACK_DATA_OUT_TERMINAL_NAME, test);
		
		// attach the terminals to the action
		actionBC.attach(dataOutB);
		actionBC.attach(ackDataOutB);

		// set the "done input" = the arbiter "done" signal
		actionBC.setDoneInput(ackDataOutB);		
		
		// attach terminals to the states
		dataC.attach(dataOutB);
		ackDataC.attach(ackDataOutB);
		
		// make the sink
		AATSink sink= new AATSink(AATGlobal.SINK_ACTION_NAME, test);
		
		// make the terminals that attach to the sink
		AATInput dataIn= new AATInput(
				AATGlobal.DATA_IN_TERMINAL_NAME, test);
		
		AATOutput ackDataIn= new AATOutput(
				AATGlobal.ACK_DATA_IN_TERMINAL_NAME, test);
		
		// attach the terminals to the sink
		sink.attach(dataIn);
		sink.attach(ackDataIn);
		
		// attach the terminals to the states
		dataC.attach(dataIn);
		ackDataC.attach(ackDataIn);
		
		// activate the ack-states to enable the simulation
		ackDataA.setActive(null);
		ackDataB.setActive(null);
		ackDataC.setActive(null);		
		
//		srcA.setDelay(new Delay(0));
//		srcB.setDelay(new Delay(100));
//		sink.setDelay(new Delay(100));
	
		
		return test;

	} // please
} // AATFactory
